#ifndef _Heap_h
#define _Heap_h

#include <algorithm> // 'std::swap'
#include <cassert>
#include <vector>

template<typename T, int default_capacity = 10 > class Heap {
private:
	T* _content; // Or 'T _content[];'
	int _n;
public:
	const int capacity;

	Heap(int capacity = default_capacity) : capacity(capacity) {
		assert(capacity > 0);
		_content = new T[capacity];
		_n = 0;
	}

	~Heap() {
		delete[] _content;
	}

	void insert(const T& t) {
		assert(_n < capacity);
		_content[_n++] = t;
		int i = _n - 1;
		for (int j = i / 2; i > 0 && _content[j] < _content[i]; j /= 2) {
			std::swap(_content[i], _content[j]);
			i = j;
		}
	}

	int size() const {
		return _n;
	}

	void sort(T result[]) const {
		assert(result);
		for (int i = 0; i < _n; i++) result[i] = _content[i];
		assert(_n > 1);
		for (int i = _n - 1; i > 0; i--) {
			std::swap(result[i], result[0]);
			int max;
			for (int j = 0; 2 * j + 1 < i; j = max) {
				if (2 * j + 2 == i) max = 2 * j + 1;
				else max = result[2 * j + 1] > result[2 * j + 2] ? 2 * j + 1 : 2 * j + 2;
				if (result[max] > result[j]) std::swap(result[max], result[j]);
			}
		}
	}
};

template<typename E>
using Heap_representation_size_type = typename E::size_type;

template<typename T, int default_capacity = 10, typename Default_representation = std::vector<T> > class Priority_queue {
private:
	Default_representation _representation;
public:

	Heap_representation_size_type<Default_representation> count() const {
		return _representation.size();
	}

	void insert(const T& t) {
		_representation.push_back(t);
		int i = _representation.size() - 1;
		for (int j = i / 2; i > 0 && _representation[j] < _representation[i]; j /= 2) {
			std::swap(_representation[i], _representation[j]);
			i = j;
		}
	}
	// To be completed here...
};

#endif
